/*------------------------ BEGIN small_blas.h BEGIN -----------------------*/
/* @file small_blas.h                                                      */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:41:00.219468                                        */
/* @brief Basic linear algebra subprogram definitions                      */
/*                                                                         */
/* A reduced set of BLAS routines optimized for Kppa-generated solvers     */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/

#ifndef __SMALL_BLAS_H__
#define __SMALL_BLAS_H__

#ifdef __cplusplus
extern "C" {
#endif


/*--------------------------------- WCOPY ---------------------------------*/
/* Copies vector x to vector y: y <= x                                     */
/* Like the BLAS {S,D}COPY(N,X,1,Y,1)                                      */
/*                                                                         */
/* @param[in]     n Vector length                                          */
/* @param[in]     x Vector x                                               */
/* @param[out]    y Vector y                                               */
/*-------------------------------------------------------------------------*/
void WCOPY(unsigned long const n, float x[], float y[]);


/*--------------------------------- WSCAL ---------------------------------*/
/* Constant times a vector: x <= alpha*x                                   */
/* Like the BLAS {S,D}SCAL(N,alpha,X,1)                                    */
/*                                                                         */
/* @param[in]     n     Vector length                                      */
/* @param[in]     alpha Scalar                                             */
/* @param[in,out] x     Vector x                                           */
/*-------------------------------------------------------------------------*/
void WSCAL(unsigned long const n, float const alpha, float x[]);


/*--------------------------------- WAXPY ---------------------------------*/
/* Constant times a vector plus a vector: y <= y + alpha*x                 */
/* Like the BLAS {S,D}AXPY(N,alpha,X,1,Y,1)                                */
/*                                                                         */
/* @param[in]     n     Vector length                                      */
/* @param[in]     alpha Scalar                                             */
/* @param[in]     x     Vector x                                           */
/* @param[in,out] y     Vector y                                           */
/*-------------------------------------------------------------------------*/
void WAXPY(unsigned long const n, float const alpha, float x[], float y[]);


/*--------------------------------- WYMXDA --------------------------------*/
/* Difference of two vectors divided by a constant: z <= (y - x) / alpha   */
/*                                                                         */
/* @param[in]     n     Vector length                                      */
/* @param[in]     x     Vector x                                           */
/* @param[in]     y     Vector y                                           */
/* @param[in]     alpha Scalar                                             */
/* @param[out]    z     Vector z                                           */
/*-------------------------------------------------------------------------*/
void WYMXDA(unsigned long const n, float x[], float y[], float const alpha,
        float z[]);



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* __SMALL_BLAS_H__ */
/*-------------------------- END small_blas.h END -------------------------*/
