/*--------------------- BEGIN small_integrate.c BEGIN ---------------------*/
/* @file small_integrate.c                                                 */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:41:00.373025                                        */
/* @brief Interface to time stepping integrator                            */
/*                                                                         */
/* Definitions of interface functions for the Kppa-generated               */
/* time stepping integrator.  These are the Kppa "entry point" routines.   */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include "small_parameters.h"
#include "small_rosenbrock.h"
#include "small_integrate.h"



/*----------------------------- GridIntegrate -----------------------------*/
/* Applies the Kppa-generated integrator to the grid                       */
/*                                                                         */
/* @param[in]     ncells Number of grid cells                              */
/* @param[in,out] conc   Species concentrations                            */
/* @param[in]     tstart Integration start time                            */
/* @param[in]     tend   Integration end time                              */
/* @param[in]     abstol Absolute integration tolerances for variable species */
/* @param[in]     reltol Relative integration tolerances for variable species */
/* @param[in,out] idata  Integer integration in/out parameters             */
/* @param[in,out] rdata  Real value integration in/out parameters          */
/* @param[out]    lastH  Last timestep in each grid cell                   */
/*-------------------------------------------------------------------------*/
void GridIntegrate(unsigned long const ncells, float conc[], float const
        tstart, float const tend, float abstol[5], float reltol[5], int
        idata[20], float rdata[20], float lastH[])
{
    /* Iterators */
    int i, j;
    /* Pointer to variable concentrations in grid cell */
    float * var;
    /* Pointer to fixed concentrations in grid cell */
    float * fix;

    #pragma omp parallel for private(i,j,var,fix) firstprivate(idata, rdata) lastprivate(idata, rdata)
    for(i=0; i<ncells; ++i)
    {
        /* Get concentrations for the current grid cell */
        var = conc + i*NSPEC;
        fix = var + NVAR;

        /* Invoke the integrator */
        Integrate(var, fix, i, tstart, tend, abstol, reltol, idata, rdata);

        /* Save the last timestep for future use */
        if(lastH) {
            lastH[i] = rdata[11];
        }

        /* Report status */
        if (idata[19] != 0) {
            printf("Kppa: INTEGRATION FAILED IN CELL %d\n", i);
            for(j=0; j<20; j++) {
                printf("Kppa: idata[%d] = %d\n", j, idata[j]);
            }
            for(j=0; j<20; j++) {
                printf("Kppa: rdata[%d] = %g\n", j, rdata[j]);
            }
        }
    }
} /* END GridIntegrate */


/*----------------------- END small_integrate.h END -----------------------*/
