/*------------------------ BEGIN small_main.c BEGIN -----------------------*/
/* @file small_main.c                                                      */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:41:00.435325                                        */
/* @brief C-language driver with restarts                                  */
/*                                                                         */
/* C-language driver with restarts                                         */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include "small_parameters.h"
#include "small_integrate.h"
#include "small_initialize.h"
#include "small_driver_parameters.h"

/*----------------------------------------------------------------------------*/
/* Lookat data                                                                */
/*----------------------------------------------------------------------------*/

FILE * lookatFile = NULL;


/*-------------------------------- Monitor --------------------------------*/
/* Reports on species concentrations as they are calculated                */
/*                                                                         */
/* @param[in]     time Integration time                                    */
/* @param[in]     step Last accepted integrator time step (H)              */
/* @param[in]     err  Error in solution vector                            */
/* @param[in]     conc Species concentrations                              */
/*-------------------------------------------------------------------------*/
void Monitor(float const time, float const step, float const err, float
        conc[])
{

    int i;
    float cspc[NMONITOR];

    cspc[0] = conc[0] / CFACTOR;
    cspc[1] = conc[1] / CFACTOR;
    cspc[2] = conc[2] / CFACTOR;
    cspc[3] = conc[3] / CFACTOR;
    cspc[4] = conc[4] / CFACTOR;
    cspc[5] = conc[6] / CFACTOR;

    printf("%5.1f%% | %9.2f | %9.3e | %9.3e ", 
           ((time-TSTART)/abs(TEND-TSTART))*100.0, 
           time, step, err);
    for(i=0; i<NMONITOR; ++i) {
        printf("| %11.4e ", cspc[i]);
    }
    printf("\n");
    fflush(stdout);
} /* END Monitor */



/*------------------------------ InitMonitor ------------------------------*/
/* Initializes the species monitor                                         */
/*                                                                         */
/* @param[in]     time Integration time                                    */
/* @param[in]     step Last accepted integrator time step (H)              */
/* @param[in]     conc Species concentrations                              */
/*-------------------------------------------------------------------------*/
void InitMonitor(float const time, float const step, float conc[])
{
    printf("\n                                    KPPA: ROSENBROCK TIME INTEGRATION OF CHEMICAL SPECIES                                     \n");
    printf("------------------------------------------------------------------------------------------------------------------------------\n");
    printf("       |     T     |     H     | Err. Norm |     O1D     |      O      |     O3      |     NO      |     NO2     |     O2     \n");

    Monitor(time, step, 0.0, conc);

} /* END InitMonitor */



/*--------------------------------- LookAt --------------------------------*/
/* Writes species concentrations to file                                   */
/*                                                                         */
/* @param[in]     time Integration time                                    */
/* @param[in]     step Last accepted integrator time step (H)              */
/* @param[in]     err  Error in solution vector                            */
/* @param[in]     conc Species concentrations                              */
/*-------------------------------------------------------------------------*/
void LookAt(float const time, float const step, float const err, float
        conc[])
{
    float data[3 + NLOOKAT];

    if(lookatFile) {
        data[0] = time;
        data[1] = step;
        data[2] = err;
        data[0] = conc[2] / CFACTOR;
        data[1] = conc[4] / CFACTOR;
        fwrite(data, sizeof(float), 3 + NLOOKAT, lookatFile);
    }
} /* END LookAt */




/*---------------------------------- main ---------------------------------*/
/* Program driver routine                                                  */
/*                                                                         */
/* @param         argc Number of command line arguments                    */
/* @param         argv Command line arguments                              */
/*-------------------------------------------------------------------------*/
int main(int argc, char* argv[])
{
    /* Integration time */
    float time;

    /* Absolute integration tolerances for variable species */
    float abstol[5];
    /* Relative integration tolerances for variable species */
    float reltol[5];

    /* Integer integration in/out parameters */
    int idata[20];
    /* Real value integration in/out parameters */
    float rdata[20];

    /* Species concentrations for all grid cells */
    float * conc;
    /* Variable concentrations in one grid cell */
    float * var;
    /* Fixed concentrations in one grid cell */
    float * fix;
   
    /* Integrator statistics */
    int nFun = 0;       /* Number of function evaluations */
    int nJac = 0;       /* Number of Jacobian evaluations */
    int nStp = 0;       /* Number of solver steps */
    int nAcc = 0;       /* Number of accepted steps */
    int nRej = 0;       /* Number of rejected steps */
    int nDec = 0;       /* Number of matrix decompositions */
    int nSol = 0;       /* Number of Ax=b solves */
    int nSng = 0;       /* Number of singular decomposition results */

    /* Working variables */
    int i;
    
    printf("Kppa: Integrating %ld grid cells, time interval [%g, %g]\n",
            NCELLS, TSTART, TEND);

    /* Initialize example grid data */
    conc = (float*)malloc(NSPEC*NCELLS*sizeof(float));
    for(i=0; i<NCELLS; ++i) {
        var = conc + i*NSPEC;
        fix = var + NVAR;
        Initialize(var, fix);
    }

    /* Set time interval */
    time = TSTART;
    
    /* Initialize vector tolerances */
    for(i=0; i<NVAR; i++) {
        abstol[i] = ATOLS;
        reltol[i] = RTOLS;
    }

    /* Initialize integration parameters to 0 = use defaults */
    memset(idata, 0, sizeof(idata));
    memset(rdata, 0, sizeof(rdata));

    /* Rosenbrock default parameters */
    idata[0] = 0;       /* System is non-autonomous: F = F(t,y) */
    idata[1] = 0;       /* Use vector tolerances */
    idata[2] = 100000;  /* Maximum number of integration steps */
    idata[3] = 5;       /* Rodas4 Rosenbrock method */
    idata[4] = 0;       /* Tolerance vectors will not be checked */

    rdata[0] = 0;       /* Integration step size lower bound: 0 recommended */
    rdata[1] = 0;       /* Integration step size upper bound: 0 recommended */
    rdata[2] = TDEL;     /* Starting integration step size */
    rdata[3] = 0.2;     /* Lower bound on step decrease factor */
    rdata[4] = 6;       /* Upper bound on step increase factor */
    rdata[5] = 0.1;     /* Step decrease factor after step rejection */
    rdata[6] = 0.9;     /* Safety factor in the computation of new step size */
    
    /* Initialize LookAt file */
    char const * fname = "small.dat";
    if(!(lookatFile = fopen(fname, "w"))) {
        printf("ERROR: Cannot create file: %s\n", fname);
        return 0;
    }

    /* Initialize species monitor */
    InitMonitor(time, rdata[2], conc);


    /* Perform time integration */
    while(time < TEND) {
        GridIntegrate(NCELLS, conc, time, TDEL + time, abstol, reltol, idata,
            rdata, NULL);


        /* Advance time */
        time += TDEL;

        /* Accumulate statistics */
        nStp += idata[12];
        nAcc += idata[13];
        nRej += idata[14];
        nSng += idata[17]; 
        nFun += idata[10];
        nJac += idata[11];
        nDec += idata[15];
        nSol += idata[16];

        /* Write species concentrations to file */
        LookAt(time, rdata[11], rdata[12], conc);

        
        /* Report species concentrations to stdout */
        Monitor(time, rdata[11], rdata[12], conc);

        
        /* Use last integration step size as the starting step size */
        rdata[2] = rdata[11];
    }

    /* Show statistics */    
    printf("Kppa: Stp=%d, Acc=%d, Rej=%d, Sng=%d, Fun=%d, Jac=%d, Dec=%d, Sol=%d\n",
           nStp, nAcc, nRej, nSng, nFun, nJac, nDec, nSol);

    /* Clean up and exit */
    if(lookatFile) {
        fflush(lookatFile);
        fclose(lookatFile);
    }
    free((void*)conc);
    return 0;

} /* END main */



/*-------------------------- END small_main.h END -------------------------*/
